@@ -55,6 +55,7 @@ namespace LongoMatch.Gui
 		public event PlaysDeletedHandler PlaysDeletedEvent;
 		public event TimeNodeChangedHandler TimeNodeChanged;
 		public event PlayCategoryChangedHandler PlayCategoryChanged;
+		public event DuplicatePlayHandler DuplicatePlay;
 		
 		/* Playlist */
 		public event RenderPlaylistHandler RenderPlaylistEvent;
@@ -287,6 +288,9 @@ namespace LongoMatch.Gui
 			/* Connect PlayListNodeAdded events */
 			playsSelection.PlayListNodeAdded += OnPlayListNodeAdded;
 			timeline.PlayListNodeAdded += OnPlayListNodeAdded;
+			
+			/* Connect duplicate plays */
+			playsSelection.DuplicatePlay += EmitDuplicatePlay;
 
 			/* Connect tags events */
 			playsSelection.TagPlay += EmitTagPlay;
@@ -352,13 +356,15 @@ namespace LongoMatch.Gui
 				playerWindow.Show();
 				
 				playercapturer.Reparent(box);
-				buttonswidget.Visible = true;
-				timeline.Visible = true;
-				if (Config.UseGameUnits) {
-					guTimeline.Visible = true;
-					gameunitstaggerwidget1.Visible = true;
-				}
 				videowidgetsbox.Visible = false;
+				if (openedProject != null) {
+					buttonswidget.Visible = true;
+					timeline.Visible = true;
+					if (Config.UseGameUnits) {
+						guTimeline.Visible = true;
+						gameunitstaggerwidget1.Visible = true;
+					}
+				}
 			} else {
 				ToggleAction action;
 				
@@ -407,6 +413,11 @@ namespace LongoMatch.Gui
 				}
 			}
 			
+			if(projectType == ProjectType.FakeCaptureProject) {
+				(downbox[videowidgetsbox] as Box.BoxChild).Expand = false;
+				(downbox[buttonswidget] as Box.BoxChild).Expand = true;
+			}
+			
 			openedProject = project;
 			this.projectType = projectType;
 			
@@ -533,14 +544,18 @@ namespace LongoMatch.Gui
 			dialog.TransientFor = (Gtk.Window)this.Toplevel;
 			res = dialog.Run();
 			dialog.Destroy();
-
+			
 			/* Close project wihtout saving */
 			if(res == (int)EndCaptureResponse.Quit) {
 				EmitCloseOpenedProject(false);
+				(downbox[videowidgetsbox] as Box.BoxChild).Expand = true;
+				(downbox[buttonswidget] as Box.BoxChild).Expand = false;
 				return true;
 			} else if(res == (int)EndCaptureResponse.Save) {
 				/* Close and save project */
 				EmitCloseOpenedProject(true);
+				(downbox[videowidgetsbox] as Box.BoxChild).Expand = true;
+				(downbox[buttonswidget] as Box.BoxChild).Expand = false;
 				return true;
 			} else
 				/* Continue with the current project */
@@ -970,6 +985,13 @@ namespace LongoMatch.Gui
 			if (KeyPressed != null)
 				KeyPressed(sender, key, modifier);
 		}
+		
+		void EmitDuplicatePlay (Play play)
+		{
+			if (DuplicatePlay != null)
+				DuplicatePlay (play);
+		}
+		
 		#endregion
 	}
 }
