@@ -89,6 +89,7 @@ namespace LongoMatch.Services
 			mainWindow.PlaysDeletedEvent += OnPlaysDeleted;
 			mainWindow.PlaySelectedEvent += OnPlaySelected;
 			mainWindow.PlayCategoryChanged += OnPlayCategoryChanged;
+			mainWindow.DuplicatePlay += OnDuplicatePlay;
 
 			/* Connect playlist events */
 			mainWindow.PlayListNodeSelectedEvent += (tn) => {selectedTimeNode = tn;};
@@ -310,6 +311,16 @@ namespace LongoMatch.Services
 			filter.Update();
 		}
 
+		void OnDuplicatePlay (Play play)
+		{
+			Play copy = Cloner.Clone (play);
+			/* The category is also serialized and desarialized */
+			copy.Category = play.Category;
+			openedProject.AddPlay (copy);
+			mainWindow.AddPlay (copy);
+			filter.Update();
+		}
+
 		protected virtual void OnSegmentClosedEvent()
 		{
 			selectedTimeNode = null;
